/*$Id: e_card.cc,v 13.27 1999/04/10 03:14:07 al Exp $ -*- C++ -*-
 * Base class for "cards" in the circuit description file
 */
#include "e_card.h"
/*--------------------------------------------------------------------------*/
//		CARD::CARD();
//		CARD::CARD(const CARD& proto);
//	char*	CARD::printlabel(int where=0)const;
//	double	CARD::probe(const CARD*,const char*);
// 	int	CARD::connects_to(int node)const;
/*--------------------------------------------------------------------------*/
double CARD::initial_voltage = NOT_INPUT;
double CARD::initial_current = NOT_INPUT;
double CARD::initial_condition = NOT_INPUT;
const int POOLSIZE = 4;
/*--------------------------------------------------------------------------*/
/* default constructor for the base class
 * IMPLICITLY called when any object of a derived class is constructed
 */
CARD::CARD()
{
  _owner = NULL;
  label[0] = '\0';
  for (int ii = 0;  ii <= NODESPERBRANCH;  ii++)
    _nodes[ii].e = _nodes[ii].t = _nodes[ii].m = INVALIDNODE;
  n = _nodes;
  _val = 0.;
  _evaliter = 0;
  _bypass = _constant = false;
}
/*--------------------------------------------------------------------------*/
/* copy constructor for the base class
 * called by copy constructor of any derived class
 * most of this only makes sense for derived objects, but it is universal
 */
CARD::CARD(const CARD& proto)
  :CKT_BASE(proto)
{
  _owner = NULL;
  strcpy(label, proto.label);
  for (int ii = 0;  ii <= NODESPERBRANCH;  ii++)
    _nodes[ii] = proto._nodes[ii];
  n = _nodes;
  _val = proto._val;
  _evaliter = 0;
  _bypass = false;
  _constant = proto._constant;
}
/*--------------------------------------------------------------------------*/
const char* CARD::printlabel(OMSTREAM where)const
{
  static char buffers[POOLSIZE][BUFLEN];
			/* CAUTION: static area overwritten every call */
  static int poolindex = 0;

  ++poolindex;
  if (poolindex >= POOLSIZE){
    poolindex = 0;
  }

  char* buffer = buffers[poolindex];
  strcpy(buffer, label);
  for (const CARD* brh = owner(); exists(brh); brh = brh->owner()){
    strcat(buffer, ".");
    strncat(buffer, brh->label, BUFLEN-strlen(buffer)-2);
  }
  strcpy(&(buffer[BUFLEN-3]), "++");
  if (any(where))
    where << buffer;
  return buffer;
}
/*--------------------------------------------------------------------------*/
/*static*/ double CARD::probe(const CARD *This, const char *what)
{
  if (exists(This)){
    return This->probe_num(what);
  }else{				/* return 0 if doesn't exist */
    return 0.0;				/* happens when optimized models */
  }					/* don't have all parts */
}
/*--------------------------------------------------------------------------*/
/* connects_to: does this part connect to this node?
 * input: a node
 * returns: how many times this part connects to it.
 * does not traverse subcircuits
 */
int CARD::connects_to(int node)const
{
  int count = 0;
  if (isdevice()){
    for (int ii = 0;  n[ii].t != INVALIDNODE;  ii++){
      if (node == n[ii].t)
        ++count;
    }
  }
  return count;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
