/*$Id: e_cardlist.h,v 13.35 1999/05/04 08:52:43 al Exp $ -*- C++ -*-
 * "card" container -- list of circuit elements
 */
#ifndef E_CARDLIST_H
#define E_CARDLIST_H
#include "md.h"
/*--------------------------------------------------------------------------*/
class CARD;
class CARD_LIST {
private:
  std::list<CARD*> cl;
public:
  // internal types
  typedef std::list<CARD*>::iterator iterator;
  typedef std::list<CARD*>::const_iterator const_iterator;
  class fat_iterator {
  private:
    CARD_LIST* _list;
    iterator   _iter;
  private:
    explicit fat_iterator()		{unreachable();}
  public:
    explicit	  fat_iterator(CARD_LIST* l, iterator i): _list(l), _iter(i) {}
    explicit	  fat_iterator(CARD_LIST* l): _list(l) {_iter=_list->begin();}
    bool	  isend()const		{return _iter == _list->end();}
    CARD*	  operator*()		{assert(!isend()); return *_iter;}
    fat_iterator& operator++()	{assert(!isend()); ++_iter; return *this;}
    fat_iterator  operator++(int)
		{assert(!isend()); fat_iterator t(*this); ++_iter; return t;}
    bool	  operator==(const fat_iterator& x)const
			{assert(_list==x._list); return (_iter==x._iter);}
    bool	  operator!=(const fat_iterator& x)const
			{assert(_list==x._list); return (_iter!=x._iter);}
    iterator	  iterator()const	{return _iter;}
    CARD_LIST*	  list()const		{return _list;}
    fat_iterator  end()const	{return fat_iterator(_list, _list->end());}
    
  };

  // status queries
  bool empty()const			{return cl.empty();}
  bool exists()const			{return !empty();}

  // return an iterator
  iterator begin()			{return cl.begin();}
  iterator end()			{return cl.end();}
  iterator find(const char* short_name);

  // return a const_iterator
  const_iterator begin()const		{return cl.begin();}
  const_iterator end()const		{return cl.end();}
  const_iterator find(const char* short_name)const;

  // add to it
  CARD_LIST& push_front(CARD* c)	{cl.push_front(c); return *this;}
  CARD_LIST& push_back(CARD* c)		{cl.push_back(c);  return *this;}
  CARD_LIST& insert(CARD_LIST::iterator i, CARD* c)
					{cl.insert(i, c);  return *this;}
  CARD_LIST& insert(CARD_LIST::fat_iterator i, CARD* c)
	{assert(i.list()==this); cl.insert(i.iterator(),c); return *this;}

  // take things out
  CARD_LIST& erase(iterator i);
  CARD_LIST& erase(fat_iterator i)	{return erase(i.iterator());}
  CARD_LIST& erase(CARD* c);

  // operations on the whole list
  CARD_LIST& expand();
  bool	     dotr();
  CARD_LIST& trload();
  CARD_LIST& trunload();
  CARD_LIST& doac();
  double     tr_review();
  CARD_LIST& tr_advance();
  CARD_LIST& tr_accept();
  CARD_LIST& destroy();

  ~CARD_LIST()				{destroy();}
public:
  static CARD_LIST card_list;
public: // d_subckt, c_getckt
  static CARD_LIST::fat_iterator putbefore;
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
