/*$Id: e_compon.cc,v 13.31 1999/04/16 07:48:59 al Exp $ -*- C++ -*-
 * Base class for elements of a circuit
 */
#include "l_stlextra.h"
#include "ap.h"
#include "e_compon.h"
/*--------------------------------------------------------------------------*/
//	const MODEL_CARD* COMPONENT_COMMON::attach_model()const;
//		COMPONENT::COMPONENT();
//		COMPONENT::COMPONENT(const COMPONENT& proto);
//	void	COMPONENT::parselabel(CS&);
//	int	COMPONENT::parsenodes(CS&,int);
//	void	COMPONENT::printnodes(int,int)const;
//	void	COMPONENT::set(............);
//	void	COMPONENT::set(............);
//	void	COMPONENT::tr_eval();
//	void	COMPONENT::ac_eval();
//	void	COMPONENT::attach_common(const COMPONENT_COMMON *);
//	void	COMPONENT::detach_common();
//	bool	COMPONENT::conv_check()const;
static	int	name2number(CS&);
static	char*	number2name(char*,int);
/*--------------------------------------------------------------------------*/
const MODEL_CARD* COMPONENT_COMMON::attach_model()const
{
  std::list<MODEL_CARD*>::const_iterator mi =
    find_ptr(root_model_list.begin(), root_model_list.end(), _modelname);
  if (mi == root_model_list.end()){
    untested();
    error(bERROR, "can't find model: %s\n", _modelname);
  }
  _model = *mi;
  assert(_model);
  return _model;
}
/*--------------------------------------------------------------------------*/
COMPONENT::COMPONENT()
  :common(0), ev(0), acbias(0), _converged(false)
{
  assert(y0.x == 0. && y0.f0 == 0. && y0.f1 == 0.);
  assert(y1 == y0);
  assert(y2 == y0);
}
/*--------------------------------------------------------------------------*/
COMPONENT::COMPONENT(const COMPONENT& p)
  :CARD(p), common(0), ev(0), acbias(0), _converged(false)
{
  attach_common(p.common);
  assert(common == p.common);
  assert(y0.x == 0. && y0.f0 == 0. && y0.f1 == 0.);
  assert(y1 == y0);
  assert(y2 == y0);
}
/*--------------------------------------------------------------------------*/
void COMPONENT::parselabel(CS& cmd)
{
  cmd.ctostr(label, LABELEN, TOKENTERM);
  label[0] = toupper(label[0]);
}
/*--------------------------------------------------------------------------*/
/* parsenodes: parse circuit connections from input string
 * n array must hold at least numnodes+1
 * fills in the rest of the array with INVALIDNODE
 * returns the number of nodes actually read
 */
int COMPONENT::parsenodes(CS& cmd, int numnodes, int start, int minfill)
{
  int paren = cmd.skiplparen();
  int count = 0;
  for (int ii = start;  ii < numnodes;  ++ii){
    n[ii].t = n[ii].e = name2number(cmd);
    if (n[ii].e != INVALIDNODE){
      count = ii+1;
    }else{
      if (ii <= minfill){
	untested();
	n[ii].t = n[ii].e = 0;
      }
    }
  }
  n[numnodes].t = n[numnodes].e = INVALIDNODE;
  paren -= cmd.skiprparen();
  if (paren != 0){
    cmd.warn(bWARNING);
  }
  return count;
}
/*--------------------------------------------------------------------------*/
/* printnodes: print a node list
 */
void COMPONENT::printnodes(OMSTREAM where, int numnodes, int start)const
{
  for (int ii = start;  ii < numnodes  &&  n[ii].e != INVALIDNODE;  ++ii){
    char name[LABELEN+1];
    where << "  " << number2name(name,n[ii].e);
  }
}
/*--------------------------------------------------------------------------*/
/* set: set values, used in model building, 2 node version
 */
COMPONENT& COMPONENT::set(const char *Label, CARD *Owner,
			  const COMPONENT_COMMON *Common, double Value,
			  node_t N0, node_t N1)
{
  strncpy(label, Label, LABELEN);
  label[LABELEN] = '\0';
  label[0] = toupper(label[0]);
  set_owner(Owner);
  setvalue(Value);
  attach_common(Common);
  y0.f1 = value();
  y0.x = y0.f0 = 0.;
  n[0] = N0;
  n[1] = N1;
  return *this;
}
/*--------------------------------------------------------------------------*/
/* set: set values, used in model building, 4 node version
 */
COMPONENT& COMPONENT::set(const char *Label, CARD *Parent,
			  const COMPONENT_COMMON *Common, double Value,
			  node_t N0, node_t N1, node_t N2, node_t N3)
{
  set(Label, Parent, Common, Value, N0, N1);
  n[2] = N2;
  n[3] = N3;
  return *this;
}
/*--------------------------------------------------------------------------*/
void COMPONENT::tr_eval()
{
  {if (common && common->has_tr_eval()){
    common->tr_eval(this);
  }else{
    y0.f1 = value();
    y0.f0 = y0.x * y0.f1;
  }}
  _converged = conv_check();
}
/*--------------------------------------------------------------------------*/
void COMPONENT::ac_eval()
{
  {if (common && common->has_ac_eval()){
    common->ac_eval(this);
  }else{
    ev = y0.f1;
  }}
}
/*--------------------------------------------------------------------------*/
void COMPONENT::attach_common(const COMPONENT_COMMON *c)
{
  {if (!c){
    // there is no new common.  probably a simple element
    detach_common();
  }else if (!common){
    // no old one, but have a new one
    common = c->attach();
  }else{ /* if (1 || *c != *common){*/
    // they are different
    detach_common();
    common = c->attach();
#if 0
  }else{
    printlabel(IO::mstdout);
    untested();
    // The one already attached matches the new one. 
    // use the old one and throw away the new one.
    // doesn't work.  always assume different as workaround
    delete c;
#endif
  }}
}
/*--------------------------------------------------------------------------*/
void COMPONENT::detach_common()
{
  if (common){
    if (common->detach() == 0){
      delete common;
      common = NULL;
    }
  }
}
/*--------------------------------------------------------------------------*/
/* conv_trace: check branch for convergence, with diagnostics
 */
bool COMPONENT::conv_trace()const
{
  bool isconverged = conv_check();

  if ((STATUS::iter[iSTEP] >= OPT::itl[OPT::TRACE]) && !isconverged){
    untested();
    error(bPICKY,"%s: non-convergence\n", printlabel());
    error(bPICKY,"(x) (%s,%s,%s)\n",
          ftos(y0.x, 0, 7, 0),
          ftos(y1.x, 0, 7, 0),
          ftos(y2.x, 0, 7, 0));
    error(bPICKY,"(f0) (%s,%s,%s)\n",
          ftos(y0.f0, 0, 7, 0),
          ftos(y1.f0, 0, 7, 0),
          ftos(y2.f0, 0, 7, 0));
    error(bPICKY,"(f1) (%s,%s,%s)\n",
	  ftos(y0.f1, 0, 7, 0),
	  ftos(y1.f1, 0, 7, 0),
	  ftos(y2.f1, 0, 7, 0));
  }else{
    untested();
  }
  return isconverged;
}
/*--------------------------------------------------------------------------*/
/* name2number: convert node name to node number
 * returns node number
 * cnt updated
 */
static int name2number(CS& cmd)
{
  int test = cmd.cursor();
  int node = cmd.ctoi();
  return  (test == cmd.cursor())  ?  INVALIDNODE  :  node;
}
/*--------------------------------------------------------------------------*/
/* number2name: convert node number to node name
 * result in name (must be big enough)
 * returns name
 */
static char* number2name(char *name, int node)
{
  sprintf(name, "%d", node);
  return name;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
