/*$Id: e_elem3.cc,v 13.43 1999/06/01 15:25:51 al Exp $ -*- C++ -*-
 * top level device functions that call the specific one and
 * standard device functions to be used where nothing else is needed
 */
#include "l_denoise.h"
#include "u_xprobe.h"
#include "e_aux.h"
#include "ap.h"
#include "e_elemnt.h"
/*--------------------------------------------------------------------------*/
//	double	  ELEMENT::tr_amps()const;
//	double	  ELEMENT::tr_outvolts()const;
//	COMPLEX	  ELEMENT::ac_outvolts()const;
//	double	  ELEMENT::probe_tr_num(const char*)const;
//	XPROBE    ELEMENT::probe_ac_ext(const char*)const;
/*--------------------------------------------------------------------------*/
double ELEMENT::tr_amps()const
{
  //return loss * tr_outvolts() + m0.f1 * tr_involts() + m0.c0;
  return dn_sum(loss * tr_outvolts(),dn_sum(m0.f1 * tr_involts(), m0.c0));
}
/*--------------------------------------------------------------------------*/
double ELEMENT::tr_outvolts()const
{
  //return n[OUT1].v0() - n[OUT2].v0();
  return dn_diff(n[OUT1].v0(), n[OUT2].v0());
}
/*--------------------------------------------------------------------------*/
COMPLEX ELEMENT::ac_outvolts()const
{
  return n[OUT1].vac() - n[OUT2].vac();
}
/*--------------------------------------------------------------------------*/
double ELEMENT::probe_tr_num(const char *what)const
{
  CS cmd(what);
  {if (cmd.pmatch("Vo")){
    return tr_outvolts();
  }else if (cmd.pmatch("VI")){
    return tr_involts();
  }else if (cmd.pmatch("I")){
    return tr_amps();
  }else if (cmd.pmatch("P")){
    return tr_amps() * tr_outvolts();
  }else if (cmd.pmatch("PD")){
    double p = tr_amps() * tr_outvolts();
    return (p > 0.) ? p : 0.;
  }else if (cmd.pmatch("PS")){
    double p = tr_amps() * tr_outvolts();
    return (p < 0.) ? -p : 0.;
  }else if (cmd.pmatch("INput")){
    untested();
    return y0.x;
  }else if (cmd.pmatch("F")){
    if (y0.f0 == LINEAR){
      untested();
      return y0.x * y0.f1;
    }else{
      return y0.f0;
    }
  }else if (cmd.pmatch("NV")){
    return value();
  }else if (cmd.pmatch("EV")){
    return y0.f1;
  }else if (cmd.pmatch("Y")){
    return m0.f1;
  }else if (cmd.pmatch("EIV")){
    untested();
    return m0.x;
  }else if (cmd.pmatch("IOFfset")){
    return m0.c0;
  }else if (cmd.pmatch("IPassive")){
    untested();
    return m0.f1 * tr_involts();
  }else if (cmd.pmatch("ILoss")){
    untested();
    return loss * tr_outvolts();
//  }else if (cmd.pmatch("DIdt")){
//    double i0  = (m0.f1  * m0.x  + m0.c0);
//    double it1 = (mt1.f1 * mt1.x + mt1.c0);
//    return  (i0 - it1) / (time0 - time1);
//  }else if (cmd.pmatch("DTNew")){
//    return timef - time0;
//  }else if (cmd.pmatch("DTOld")){
//    return time0 - time1;
//  }else if (cmd.pmatch("TIMEF")){
//    return timef;
//  }else if (cmd.pmatch("TIME")){
//    return time0;
//  }else if (cmd.pmatch("TIMEO")){
//    return time1;
  }else if (cmd.pmatch("R")){
    return (m0.f1!=0.) ? 1/m0.f1 : DBL_MAX;
  }else if (cmd.pmatch("Z")){
    return port_impedance(n[OUT1], n[OUT2], lu, (m0.f1+loss));
  }else if (cmd.pmatch("ZRAW")){
    return port_impedance(n[OUT1], n[OUT2], lu, 0.);
  }else{
    untested();
    return NOT_VALID;
  }}
}
/*--------------------------------------------------------------------------*/
XPROBE ELEMENT::probe_ac_ext(const char *what)const
{
  CS cmd(what);
  COMPLEX admittance = (issource()) ? COMPLEX(0.) : acg;
  COMPLEX source =     (issource()) ? acg : COMPLEX(0.);

  {if (cmd.pmatch("V")){			/* volts */
    return XPROBE(ac_outvolts());
  }else if (cmd.pmatch("I")){			/* amps */
    return XPROBE(ac_involts() * admittance + source + ac_outvolts() * loss);
  }else if (cmd.pmatch("P")){			/* complex "power" */
    COMPLEX i = ac_involts() * admittance + source + ac_outvolts() * loss;
    return XPROBE(ac_outvolts() * std::conj(i), mtREAL, 10.);
  }else if (cmd.pmatch("NV")){			/* nominal value */
    untested();
    return XPROBE(value());
  }else if (cmd.pmatch("EV")){			/* effective value */
    return XPROBE(ev);
  }else if (cmd.pmatch("Y")){			/* admittance */
    untested();
    return XPROBE(admittance, mtREAL);
  }else if (cmd.pmatch("R")){			/* complex "resistance" */
    {if (admittance == 0.){
      untested();
      return XPROBE(DBL_MAX);
    }else{
      untested();
      return XPROBE(1. / admittance);
    }}
  }else if (cmd.pmatch("Z")){			/* port impedance */
    return XPROBE(port_impedance(n[OUT1], n[OUT2], acx, admittance));
  }else if (cmd.pmatch("ZRAW")){		/* port impedance, raw */
    return XPROBE(port_impedance(n[OUT1], n[OUT2], acx, COMPLEX(0.)));
  }else{ 					/* bad parameter */
    return XPROBE();
  }}
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
