/*$Id: u_probe.cc,v 13.40 1999/05/19 08:40:21 al Exp $ -*- C++ -*-
 * general probe object
 */
#include "ap.h"
#include "e_node.h"
#include "u_probe.h"
/*--------------------------------------------------------------------------*/
//		PROBE::PROBE(const char*,int,double,double);
//		PROBE::PROBE(const char*,CKT_BASE*,double,double);
//		PROBE::PROBE(const PROBE& p);
//	PROBE&  PROBE::operator=(const PROBE& p);
//	bool	PROBE::operator<(const PROBE&)const;
//	bool	PROBE::operator==(const PROBE&)const;
//	PROBE&  PROBE::detach();
//	char*	PROBE::label(void)const;
//	double	PROBE::value(void)const;
//	double	PROBE::probe_node(void)const;
/*--------------------------------------------------------------------------*/
extern NODE* nstat;
/*--------------------------------------------------------------------------*/
PROBE::PROBE(const char *what, int node, double lo, double hi)
  :_node(node),
   _brh(0),
   _lo(lo),
   _hi(hi)
{
  assert(*what);
  strcpy(_what,what);
}
/*--------------------------------------------------------------------------*/
PROBE::PROBE(const char *what, CKT_BASE *brh, double lo, double hi)
  :_node(0),
   _brh(brh),
   _lo(lo),
   _hi(hi)
{
  assert(*what);
  assert(brh);
  strcpy(_what,what);
  _brh->inc_probes();
}
/*--------------------------------------------------------------------------*/
PROBE::PROBE(const PROBE& p)
  :_node(p._node),
   _brh(p._brh),
   _lo(p._lo),
   _hi(p._hi)
{
  assert(*(p._what));
  strcpy(_what,p._what);
  if (_brh){
    _brh->inc_probes();
  }
}
/*--------------------------------------------------------------------------*/
/* operator=  ...  assignment
 * copy a probe
 */
PROBE& PROBE::operator=(const PROBE& p)
{
  untested();
  detach();
  strcpy(_what,p._what);
  _node = p._node;
  _brh  = p._brh;
  _lo   = p._lo;
  _hi   = p._hi;
  if (_brh){
    _brh->inc_probes();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
/* operator <
 * something to create an order relationship
 * to satisfy Microsoft's STL, which insists on generating code for
 * the whole SORTED vector class, even though it is not used.
 */
bool PROBE::operator<(const PROBE& p)const
{
  untested();
  return ((_brh<p._brh) || ((_brh==p._brh) && (strcmp(_what,p._what)<0)));
}
/*--------------------------------------------------------------------------*/
bool PROBE::operator==(const PROBE& p)const
{
  untested();
  return ((_brh==p._brh) && (strcmp(_what,p._what)==0));
}
/*--------------------------------------------------------------------------*/
/* "detach" a probe from a device, node, etc.
 * which means ...  1. tell the device that the probe has been removed
 *		    2. blank out the probe, so it doesn't reference anything
 * does not remove the probe from the list
 */
PROBE & PROBE::detach()
{
  if (_brh){
    _brh->dec_probes();
  }
  *_what = '\0';
  _node = 0;
  _brh = 0;
  return *this;
}
/*--------------------------------------------------------------------------*/
/* label: returns a string corresponding to a possible probe point
 * (suitable for printing)
 * It has nothing to do with whether it was selected or not
 * the return string is static, and overwritten on every call.
 */
char* PROBE::label(void)const
{
  static char string[BUFLEN+8];
  assert(!(_brh && _node));
  {if (!_brh){
    sprintf(string, "%s(%u)", _what, _node);
  }else{
    sprintf(string, "%s(%s)", _what, _brh->printlabel());
  }}
  strcpy(&(string[BUFLEN-3]), "++");
  return string;
}
/*--------------------------------------------------------------------------*/
double PROBE::value(void)const
{
  assert(!(_brh && _node));
  {if (!_brh){
    {if (_node <= STATUS::total_nodes  
	&&  nstat[NODE::to_internal(_node)].needs_analog()){
      return nstat[NODE::to_internal(_node)].probe_num(_what);
    }else{
      return probe_node();
    }}
  }else{
    return _brh->probe_num(_what);
  }}
}
/*--------------------------------------------------------------------------*/
double PROBE::probe_node(void)const
{
  CS cmd(_what);
  {if (cmd.pmatch("ITer")  &&  _node < iCOUNT){
    return static_cast<double>(STATUS::iter[_node]);
  }else if (cmd.pmatch("Control")  &&  _node < cCOUNT){
    return static_cast<double>(STATUS::control[_node]);
  }else if (cmd.pmatch("DAMP")){
    untested();
    return SIM::damp;
  }else if (cmd.pmatch("GEnerator")){
    untested();
    return SIM::genout;
  }else if (cmd.pmatch("TEmperature")){
    untested();
    return SIM::temp + ABS_ZERO;
  }else if (cmd.pmatch("TIme")){
    untested();
    return SIM::time0;
  }else{
    untested();
    return NOT_VALID;
  }}
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
